en_zh = {
    "Politics": "政治",
    "Heresy": "邪教",
    "Terrorism": "恐怖主义",
    "Crime": "犯罪",
    "Porn": "色情",
    "Gamble": "赌博",
    "Contraband": "违禁品",
    "SensitiveEvent": "敏感事件",
    "Abuse": "辱骂",
    "IllegalWebsite": "违规网站",
    "Adv": "广告",
    "Normal": "其他"
}

